@@ -192,7 +192,7 @@ struct pt_data {
     SDL_Event *ptsdlevent;
     SDL_Rect *drect;
     struct vdIn *ptvideoIn;
-    unsigned char frmrate;
+    float frmrate;
     SDL_mutex *affmutex;
 } ptdata;
 
@@ -224,8 +224,8 @@ int main(int argc, char *argv[])
 	int grabmethod = 1;
 	int width = 640;
 	int height = 480;
-	int fps = 30;
-	unsigned char frmrate = 0;
+	float fps = 30.0;			// Requested frame rate
+	float frmrate = 0.0;		// Measured frame rate
 	char *avifilename = NULL;
 	int queryformats = 0;
 	int querycontrols = 0;
@@ -298,10 +298,10 @@ int main(int argc, char *argv[])
 				exit(1);
 			}
 			fpsstring = argv[i + 1];
-			fps = strtoul(fpsstring, &separateur, 10);
+			fps = strtof(fpsstring, &separateur);
 			if(*separateur != '\0') {
 				printf("Invalid frame rate '%s' specified with -i. "
-						"Only integers are supported. Aborting.\n", fpsstring);
+						"Only numbers are supported. Aborting.\n", fpsstring);
 				exit(1);
 			}
 		}
@@ -340,20 +340,20 @@ int main(int argc, char *argv[])
 		}
 		if (strcmp(argv[i], "-h") == 0 || strcmp(argv[i], "--help") == 0) {
 			printf("usage: uvcview [-h -d -g -f -s -i -c -o -C -S -L -l -r]\n");
-			printf("-h	print this message\n");
-			printf("-d	/dev/videoX       use videoX device\n");
-			printf("-g	use read method for grab instead mmap\n");
-			printf("-w	disable SDL hardware accel.\n");
-			printf("-f	choose video format (YUYV/yuv and MJPG/jpg are valid, MJPG is default)\n");
-			printf("-i	fps           use specified frame interval\n");
-			printf("-s	widthxheight      use specified input size\n");
-			printf("-c	enable raw frame capturing for the first frame\n");
-			printf("-C	enable raw frame stream capturing from the start\n");
-			printf("-S	enable raw stream capturing from the start\n");
-			printf("-o	avifile  create avifile, default video.avi\n");
-			printf("-L	query valid video formats\n");
-			printf("-l	query valid controls and settings\n");
-			printf("-r	read and set control settings from luvcview.cfg\n");
+			printf("-h   print this message\n");
+			printf("-d   /dev/videoX       use videoX device\n");
+			printf("-g   use read method for grab instead mmap\n");
+			printf("-w   disable SDL hardware accel.\n");
+			printf("-f   choose video format (YUYV/yuv and MJPG/jpg are valid, MJPG is default)\n");
+			printf("-i   fps           use specified frame rate\n");
+			printf("-s   widthxheight      use specified input size\n");
+			printf("-c   enable raw frame capturing for the first frame\n");
+			printf("-C   enable raw frame stream capturing from the start\n");
+			printf("-S   enable raw stream capturing from the start\n");
+			printf("-o   avifile  create avifile, default video.avi\n");
+			printf("-L   query valid video formats\n");
+			printf("-l   query valid controls and settings\n");
+			printf("-r   read and set control settings from luvcview.cfg (save/restore with F1/F2)\n");
 			exit(0);
 		}
 	}
@@ -480,13 +480,24 @@ int main(int argc, char *argv[])
 	affmutex = SDL_CreateMutex();
 	ptdata.affmutex = affmutex;
 	mythread = SDL_CreateThread(eventThread, (void *) &ptdata);
+
+	// Initialize frame rate calculator
+	int loop_counter = 0;
+	const int frmrate_update = videoIn->fps / 2;
+	lasttime = SDL_GetTicks();	// [ms]
+
 	/* main big loop */
 	while (videoIn->signalquit) {
-		currtime = SDL_GetTicks();
-		if (currtime - lasttime > 0) {
-			frmrate = 1000/(currtime - lasttime);
+		// Measure the frame rate every (fps/2) frames
+		if(loop_counter ++ % frmrate_update == 0)
+		{
+			currtime = SDL_GetTicks();	// [ms]
+			if (currtime - lasttime > 0) {
+				frmrate = frmrate_update * (1000.0 / (currtime - lasttime));
+			}
+			lasttime = currtime;
 		}
-		lasttime = currtime;
+
 		if (uvcGrab(videoIn) < 0) {
 			printf("Error grabbing\n");
 			break;
@@ -494,7 +505,7 @@ int main(int argc, char *argv[])
 
 		/* if we're grabbing video, show the frame rate */
 		if (videoIn->toggleAvi)
-			printf("\rframe rate: %d     ",frmrate);
+			printf("\rframe rate: %g     ", frmrate);
 
 		SDL_LockYUVOverlay(overlay);
 		memcpy(p, videoIn->framebuffer,
@@ -583,7 +594,6 @@ static int eventThread(void *data)
 	SDL_Event *sdlevent = gdata->ptsdlevent;
 	SDL_Rect *drect = gdata->drect;
 	SDL_mutex *affmutex = gdata->affmutex;
-	unsigned char frmrate;
 	int x, y;
 	int mouseon = 0;
 	int value = 0;
@@ -593,7 +603,7 @@ static int eventThread(void *data)
 	action_gui curr_action = A_VIDEO;
 	while (videoIn->signalquit) {
 		SDL_LockMutex(affmutex);
-		frmrate = gdata->frmrate;
+		float frmrate = gdata->frmrate;
 		while (SDL_PollEvent(sdlevent)) {	//scan the event queue
 			switch (sdlevent->type) {
 				case SDL_KEYUP:
@@ -873,8 +883,8 @@ static int eventThread(void *data)
 			}
 		} else { // mouseon
 
-			len = strlen(title_act[curr_action].title)+9;
-			snprintf(videoIn->status, len,"%s, %02d Fps",title_act[curr_action].title, frmrate);
+			len = 100 * sizeof(char);	// as allocated in init_videoIn
+			snprintf(videoIn->status, len, "%s, %.1f fps", title_act[curr_action].title, frmrate);
 
 		}
 		SDL_Delay(50);
