@@ -183,6 +183,7 @@ static int readtables(int till, int *isDHT)
     for (;;) {
 	if (getbyte() != 0xff)
 	    return -1;
+nextbyte:
 	if ((m = getbyte()) == till)
 	    break;
 
@@ -239,6 +240,9 @@ static int readtables(int till, int *isDHT)
 	    l = getword();
 	    info.dri = getword();
 	    break;
+	case 0xff:
+	    goto nextbyte;
+	    break;
 
 	default:
 	    l = getword();
