@@ -22,6 +22,8 @@
 *******************************************************************************/
 
 #include <stdlib.h>
+#include <math.h>
+#include <float.h>
 
 #include "v4l2uvc.h"
 #include "utils.h"
@@ -37,6 +39,30 @@ static int debug = 0;
 
 static int init_v4l2(struct vdIn *vd);
 
+static int float_to_fraction_recursive(double f, double p, int *num, int *den)
+{
+	int whole = (int)f;
+	f = fabs(f - whole);
+
+	if(f > p) {
+		int n, d;
+		int a = float_to_fraction_recursive(1 / f, p + p / f, &n, &d);
+		*num = d;
+		*den = d * a + n;
+	}
+	else {
+		*num = 0;
+		*den = 1;
+	}
+	return whole;
+}
+
+static void float_to_fraction(float f, int *num, int *den)
+{
+	int whole = float_to_fraction_recursive(f, FLT_EPSILON, num, den);
+	*num += whole * *den;
+}
+
 int check_videoIn(struct vdIn *vd, char *device)
 {
 	int ret;
@@ -74,7 +100,7 @@ fatal:
 	return 0;
 }
 int
-init_videoIn(struct vdIn *vd, char *device, int width, int height, int fps,
+init_videoIn(struct vdIn *vd, char *device, int width, int height, float fps,
 	     int format, int grabmethod, char *avifilename)
 {
    int ret = -1;
@@ -418,25 +444,30 @@ static int init_v4l2(struct vdIn *vd)
 	setfps=(struct v4l2_streamparm *) calloc(1, sizeof(struct v4l2_streamparm));
 	memset(setfps, 0, sizeof(struct v4l2_streamparm));
 	setfps->type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
-	setfps->parm.capture.timeperframe.numerator=1;
-	setfps->parm.capture.timeperframe.denominator=vd->fps;
-	ret = ioctl(vd->fd, VIDIOC_S_PARM, setfps); 
+
+	// Convert the frame rate into a fraction for V4L2
+	int n = 0, d = 0;
+	float_to_fraction(vd->fps, &n, &d);
+	setfps->parm.capture.timeperframe.numerator = d;
+	setfps->parm.capture.timeperframe.denominator = n;
+
+	ret = ioctl(vd->fd, VIDIOC_S_PARM, setfps);
 	if(ret == -1) {
 		perror("Unable to set frame rate");
 		goto fatal;
 	}
 	ret = ioctl(vd->fd, VIDIOC_G_PARM, setfps); 
 	if(ret == 0) {
-		if (setfps->parm.capture.timeperframe.numerator != 1 ||
-			setfps->parm.capture.timeperframe.denominator != vd->fps) {
-			printf("  Frame rate:   %u/%u fps (requested frame rate %u fps is "
+		float confirmed_fps = (float)setfps->parm.capture.timeperframe.denominator / (float)setfps->parm.capture.timeperframe.numerator;
+		if (confirmed_fps != (float)n / (float)d) {
+			printf("  Frame rate:   %g fps (requested frame rate %g fps is "
 				"not supported by device)\n",
-				setfps->parm.capture.timeperframe.denominator,
-				setfps->parm.capture.timeperframe.numerator,
+				confirmed_fps,
 				vd->fps);
+			vd->fps = confirmed_fps;
 		}
 		else {
-			printf("  Frame rate:   %d fps\n", vd->fps);
+			printf("  Frame rate:   %g fps\n", vd->fps);
 		}
 	}
 	else {
